# Benchmark sparse-dot-topn

Minor benchmark sweet to compare against Scipy's sparse matrix multiplication

## Dependencies

```shell
pip install richbench pandas scikit-learn sparse-dot-topn
```

## Data

List of company names from the EDGAR company database is used as a reference set.
You can download it at: `https://www.kaggle.com/datasets/dattapiy/sec-edgar-companies-list?select=sec__edgar_company_info.csv`.
The script expects the csv with the name: `sec__edgar_company_info.csv` or you can set the filename on line 19 in `bench_scipy_csr.py`.
A and B are word-based TF-IDF matrices over the company names.

You can change the number of rows in the matrices by setting `N_ROWS` in `bench_scipy_csr.py` on line 18.

### Reference

```shell
richbench /bench --repeat 30 --times 1
```

## Results

### Scipy 1.12.0 vs sparse-dot-topn v1.0.0 

Time is `A.dot(B)` with shapes `(20_000, 193190) x (193190, 20_000)` repeated 30 times.

**Apple Macbook Pro | M2 Pro | 32GB RAM**

| Benchmark               | top_n | n_threads | Min     | Max     | Mean    | Min (+)       | Max (+)       | Mean (+)      |
| :-----------------------| :---: | :-------: | :------ | :------ | :------ | :------------ | :------------ | :------------ |
| Scipy vs sp_matmul      |       | 1         | 0.165   | 0.192   | 0.167   | 0.165 (1.0x)  | 0.175 (1.1x)  | 0.167 (1.0x)  |
| Scipy vs sp_matmul      |       | 2         | 0.165   | 0.169   | 0.166   | 0.087 (1.9x)  | 0.091 (1.9x)  | 0.088 (1.9x)  |
| Scipy vs sp_matmul      |       | 4         | 0.165   | 0.172   | 0.166   | 0.051 (3.3x)  | 0.053 (3.2x)  | 0.052 (3.2x)  |
| Scipy vs sp_matmul      |       | 8         | 0.165   | 0.171   | 0.166   | 0.032 (5.1x)  | 0.035 (5.0x)  | 0.033 (5.0x)  |
| Scipy vs sp_matmul_topn | 10    | 1         | 0.165   | 0.181   | 0.166   | 0.163 (1.0x)  | 0.168 (1.1x)  | 0.164 (1.0x)  |
| Scipy vs sp_matmul_topn | 20    | 1         | 0.165   | 0.177   | 0.167   | 0.186 (-1.1x) | 0.192 (-1.1x) | 0.188 (-1.1x) |
| Scipy vs sp_matmul_topn | 30    | 1         | 0.165   | 0.181   | 0.167   | 0.199 (-1.2x) | 0.204 (-1.1x) | 0.200 (-1.2x) |
| Scipy vs sp_matmul_topn | 100   | 1         | 0.166   | 0.173   | 0.167   | 0.324 (-1.9x) | 0.337 (-1.9x) | 0.326 (-2.0x) |
| Scipy vs sp_matmul_topn | 1000  | 1         | 0.166   | 0.171   | 0.167   | 1.033 (-6.2x) | 1.054 (-6.2x) | 1.036 (-6.2x) |
| Scipy vs sp_matmul_topn | 10    | 2         | 0.165   | 0.171   | 0.166   | 0.077 (2.1x)  | 0.080 (2.1x)  | 0.078 (2.1x)  |
| Scipy vs sp_matmul_topn | 20    | 2         | 0.167   | 0.179   | 0.168   | 0.090 (1.9x)  | 0.092 (2.0x)  | 0.091 (1.9x)  |
| Scipy vs sp_matmul_topn | 30    | 2         | 0.165   | 0.185   | 0.168   | 0.097 (1.7x)  | 0.105 (1.8x)  | 0.099 (1.7x)  |
| Scipy vs sp_matmul_topn | 100   | 2         | 0.165   | 0.172   | 0.166   | 0.166 (-1.0x) | 0.170 (1.0x)  | 0.168 (-1.0x) |
| Scipy vs sp_matmul_topn | 1000  | 2         | 0.166   | 0.171   | 0.166   | 0.575 (-3.5x) | 0.593 (-3.5x) | 0.578 (-3.5x) |
| Scipy vs sp_matmul_topn | 10    | 4         | 0.166   | 0.174   | 0.167   | 0.042 (4.0x)  | 0.043 (4.0x)  | 0.042 (3.9x)  |
| Scipy vs sp_matmul_topn | 20    | 4         | 0.165   | 0.174   | 0.166   | 0.048 (3.4x)  | 0.053 (3.3x)  | 0.050 (3.3x)  |
| Scipy vs sp_matmul_topn | 30    | 4         | 0.166   | 0.181   | 0.168   | 0.052 (3.2x)  | 0.054 (3.4x)  | 0.053 (3.2x)  |
| Scipy vs sp_matmul_topn | 100   | 4         | 0.166   | 0.176   | 0.168   | 0.089 (1.9x)  | 0.092 (1.9x)  | 0.090 (1.9x)  |
| Scipy vs sp_matmul_topn | 1000  | 4         | 0.166   | 0.171   | 0.167   | 0.318 (-1.9x) | 0.326 (-1.9x) | 0.320 (-1.9x) |
| Scipy vs sp_matmul_topn | 10    | 8         | 0.166   | 0.174   | 0.167   | 0.024 (6.9x)  | 0.025 (6.9x)  | 0.025 (6.8x)  |
| Scipy vs sp_matmul_topn | 20    | 8         | 0.165   | 0.179   | 0.167   | 0.028 (5.9x)  | 0.031 (5.8x)  | 0.028 (5.9x)  |
| Scipy vs sp_matmul_topn | 30    | 8         | 0.166   | 0.173   | 0.167   | 0.030 (5.5x)  | 0.031 (5.6x)  | 0.030 (5.5x)  |
| Scipy vs sp_matmul_topn | 100   | 8         | 0.166   | 0.174   | 0.167   | 0.051 (3.3x)  | 0.052 (3.3x)  | 0.051 (3.3x)  |
| Scipy vs sp_matmul_topn | 1000  | 8         | 0.165   | 0.168   | 0.166   | 0.191 (-1.2x) | 0.196 (-1.2x) | 0.193 (-1.2x) |

**Apple Macbook Pro | 2,3 GHz 8-Core Intel Core i9 | 16GB RAM**

| Benchmark               | top_n | n_threads | Min     | Max     | Mean    | Min (+)         | Max (+)         | Mean (+)        |
| :-----------------------| :---: | :-------: | :------ | :------ | :------ | :-------------- | :-------------- | :-------------- |
| Scipy vs sp_matmul      |       | 1         | 0.501   | 0.512   | 0.505   | 0.505 (-1.0x)   | 0.515 (-1.0x)   | 0.508 (-1.0x)   |
| Scipy vs sp_matmul      |       | 2         | 0.503   | 0.515   | 0.507   | 0.307 (1.6x)    | 0.316 (1.6x)    | 0.312 (1.6x)    |
| Scipy vs sp_matmul      |       | 4         | 0.503   | 0.518   | 0.506   | 0.205 (2.5x)    | 0.217 (2.4x)    | 0.210 (2.4x)    |
| Scipy vs sp_matmul      |       | 8         | 0.501   | 0.514   | 0.505   | 0.146 (3.4x)    | 0.150 (3.4x)    | 0.148 (3.4x)    |
| Scipy vs sp_matmul_topn | 10    | 1         | 0.505   | 0.518   | 0.509   | 0.295 (1.7x)    | 0.299 (1.7x)    | 0.297 (1.7x)    |
| Scipy vs sp_matmul_topn | 20    | 1         | 0.505   | 0.512   | 0.508   | 0.342 (1.5x)    | 0.354 (1.4x)    | 0.344 (1.5x)    |
| Scipy vs sp_matmul_topn | 30    | 1         | 0.506   | 0.515   | 0.509   | 0.385 (1.3x)    | 0.389 (1.3x)    | 0.386 (1.3x)    |
| Scipy vs sp_matmul_topn | 100   | 1         | 0.504   | 0.540   | 0.508   | 0.676 (-1.3x)   | 0.690 (-1.3x)   | 0.678 (-1.3x)   |
| Scipy vs sp_matmul_topn | 1000  | 1         | 0.506   | 0.519   | 0.509   | 2.738 (-5.4x)   | 2.828 (-5.4x)   | 2.744 (-5.4x)   |
| Scipy vs sp_matmul_topn | 10    | 2         | 0.505   | 0.516   | 0.510   | 0.131 (3.9x)    | 0.137 (3.8x)    | 0.133 (3.8x)    |
| Scipy vs sp_matmul_topn | 20    | 2         | 0.503   | 0.512   | 0.507   | 0.155 (3.2x)    | 0.162 (3.2x)    | 0.158 (3.2x)    |
| Scipy vs sp_matmul_topn | 30    | 2         | 0.511   | 0.529   | 0.516   | 0.180 (2.8x)    | 0.190 (2.8x)    | 0.183 (2.8x)    |
| Scipy vs sp_matmul_topn | 100   | 2         | 0.502   | 0.521   | 0.508   | 0.350 (1.4x)    | 0.371 (1.4x)    | 0.353 (1.4x)    |
| Scipy vs sp_matmul_topn | 1000  | 2         | 0.505   | 0.521   | 0.511   | 1.437 (-2.8x)   | 1.525 (-2.9x)   | 1.441 (-2.8x)   |
| Scipy vs sp_matmul_topn | 10    | 4         | 0.504   | 0.515   | 0.508   | 0.069 (7.3x)    | 0.086 (6.0x)    | 0.070 (7.2x)    |
| Scipy vs sp_matmul_topn | 20    | 4         | 0.503   | 0.513   | 0.507   | 0.082 (6.2x)    | 0.086 (5.9x)    | 0.083 (6.1x)    |
| Scipy vs sp_matmul_topn | 30    | 4         | 0.506   | 0.528   | 0.509   | 0.095 (5.3x)    | 0.099 (5.3x)    | 0.096 (5.3x)    |
| Scipy vs sp_matmul_topn | 100   | 4         | 0.503   | 0.594   | 0.510   | 0.186 (2.7x)    | 0.202 (2.9x)    | 0.188 (2.7x)    |
| Scipy vs sp_matmul_topn | 1000  | 4         | 0.505   | 0.558   | 0.511   | 0.788 (-1.6x)   | 0.870 (-1.6x)   | 0.793 (-1.6x)   |
| Scipy vs sp_matmul_topn | 10    | 8         | 0.507   | 0.515   | 0.510   | 0.040 (12.7x)   | 0.050 (10.2x)   | 0.041 (12.3x)   |
| Scipy vs sp_matmul_topn | 20    | 8         | 0.503   | 0.518   | 0.508   | 0.048 (10.5x)   | 0.050 (10.3x)   | 0.048 (10.5x)   |
| Scipy vs sp_matmul_topn | 30    | 8         | 0.504   | 0.520   | 0.507   | 0.055 (9.1x)    | 0.062 (8.3x)    | 0.057 (8.9x)    |
| Scipy vs sp_matmul_topn | 100   | 8         | 0.503   | 0.679   | 0.514   | 0.114 (4.4x)    | 0.118 (5.8x)    | 0.115 (4.5x)    |
| Scipy vs sp_matmul_topn | 1000  | 8         | 0.505   | 2.600   | 0.588   | 0.531 (-1.1x)   | 0.543 (4.8x)    | 0.534 (1.1x)    |

 
### sparse-dot-topn v0.3.6 vs sparse-dot-topn v1.0.0 

Time is `A.dot(B)` with shapes `(100_000, 193190) x (193190, 100_000)` repeated 30 times.

Threshold is `None` indicates that we do not set a threshold for `v1.0.0` and pre-compute the number of non-zero entries in C.
We do not do this for `v0.3.6`, for which `None` and `0.0` are equivalent. 

**Apple Macbook Pro | M2 Pro | 32GB RAM**
 
| Benchmark       | top_n | n_threads | threshold | Min     | Max     | Mean    | Min (+)         | Max (+)         | Mean (+)        |
| :-------------- | :---: | :-------: | :-------: | :------ | :------ | :------ | :-------------- | :-------------- | :-------------- |
| v0.3.6 vs 1.0.0 | 10    | 1         | None      | 3.942   | 4.107   | 4.038   | 4.344 (-1.1x)   | 4.475 (-1.1x)   | 4.402 (-1.1x)   |
| v0.3.6 vs 1.0.0 | 20    | 1         | None      | 4.082   | 4.372   | 4.149   | 4.503 (-1.1x)   | 4.664 (-1.1x)   | 4.618 (-1.1x)   |
| v0.3.6 vs 1.0.0 | 30    | 1         | None      | 4.213   | 4.362   | 4.268   | 4.586 (-1.1x)   | 4.717 (-1.1x)   | 4.638 (-1.1x)   |
| v0.3.6 vs 1.0.0 | 100   | 1         | None      | 5.225   | 5.366   | 5.295   | 5.613 (-1.1x)   | 5.736 (-1.1x)   | 5.674 (-1.1x)   |
| v0.3.6 vs 1.0.0 | 1000  | 1         | None      | 14.720  | 14.993  | 14.777  | 13.309 (1.1x)   | 13.381 (1.1x)   | 13.337 (1.1x)   |
| v0.3.6 vs 1.0.0 | 10    | 1         | 0.0       | 4.017   | 4.070   | 4.042   | 3.579 (1.1x)    | 3.714 (1.1x)    | 3.684 (1.1x)    |
| v0.3.6 vs 1.0.0 | 20    | 1         | 0.0       | 4.079   | 4.224   | 4.140   | 3.737 (1.1x)    | 3.897 (1.1x)    | 3.796 (1.1x)    |
| v0.3.6 vs 1.0.0 | 30    | 1         | 0.0       | 4.221   | 4.366   | 4.276   | 3.821 (1.1x)    | 3.943 (1.1x)    | 3.883 (1.1x)    |
| v0.3.6 vs 1.0.0 | 100   | 1         | 0.0       | 5.223   | 5.338   | 5.282   | 4.859 (1.1x)    | 4.992 (1.1x)    | 4.932 (1.1x)    |
| v0.3.6 vs 1.0.0 | 1000  | 1         | 0.0       | 14.776  | 14.878  | 14.845  | 12.531 (1.2x)   | 12.596 (1.2x)   | 12.562 (1.2x)   |
| v0.3.6 vs 1.0.0 | 10    | 2         | 0.0       | 2.124   | 2.583   | 2.508   | 1.874 (1.1x)    | 1.963 (1.3x)    | 1.902 (1.3x)    |
| v0.3.6 vs 1.0.0 | 20    | 2         | 0.0       | 2.506   | 2.652   | 2.591   | 1.959 (1.3x)    | 2.027 (1.3x)    | 1.981 (1.3x)    |
| v0.3.6 vs 1.0.0 | 30    | 2         | 0.0       | 2.179   | 2.668   | 2.571   | 2.002 (1.1x)    | 2.067 (1.3x)    | 2.022 (1.3x)    |
| v0.3.6 vs 1.0.0 | 100   | 2         | 0.0       | 2.579   | 3.010   | 2.966   | 2.538 (1.0x)    | 2.601 (1.2x)    | 2.566 (1.2x)    |
| v0.3.6 vs 1.0.0 | 1000  | 2         | 0.0       | 5.941   | 6.063   | 6.012   | 6.624 (-1.1x)   | 6.673 (-1.1x)   | 6.647 (-1.1x)   |
| v0.3.6 vs 1.0.0 | 10    | 4         | 0.0       | 1.519   | 1.605   | 1.559   | 0.968 (1.6x)    | 1.006 (1.6x)    | 0.992 (1.6x)    |
| v0.3.6 vs 1.0.0 | 20    | 4         | 0.0       | 1.540   | 1.645   | 1.584   | 1.010 (1.5x)    | 1.046 (1.6x)    | 1.026 (1.5x)    |
| v0.3.6 vs 1.0.0 | 30    | 4         | 0.0       | 1.558   | 1.696   | 1.607   | 1.034 (1.5x)    | 1.065 (1.6x)    | 1.045 (1.5x)    |
| v0.3.6 vs 1.0.0 | 100   | 4         | 0.0       | 1.698   | 1.839   | 1.739   | 1.315 (1.3x)    | 1.348 (1.4x)    | 1.327 (1.3x)    |
| v0.3.6 vs 1.0.0 | 1000  | 4         | 0.0       | 3.164   | 3.258   | 3.192   | 3.483 (-1.1x)   | 3.496 (-1.1x)   | 3.489 (-1.1x)   |
| v0.3.6 vs 1.0.0 | 10    | 8         | 0.0       | 1.117   | 1.288   | 1.215   | 0.487 (2.3x)    | 0.508 (2.5x)    | 0.501 (2.4x)    |
| v0.3.6 vs 1.0.0 | 20    | 8         | 0.0       | 1.118   | 1.280   | 1.206   | 0.509 (2.2x)    | 0.527 (2.4x)    | 0.516 (2.3x)    |
| v0.3.6 vs 1.0.0 | 30    | 8         | 0.0       | 1.125   | 1.292   | 1.210   | 0.522 (2.2x)    | 0.541 (2.4x)    | 0.527 (2.3x)    |
| v0.3.6 vs 1.0.0 | 100   | 8         | 0.0       | 1.152   | 1.323   | 1.218   | 0.666 (1.7x)    | 0.686 (1.9x)    | 0.673 (1.8x)    |
| v0.3.6 vs 1.0.0 | 1000  | 8         | 0.0       | 1.833   | 1.893   | 1.850   | 1.810 (1.0x)    | 1.829 (1.0x)    | 1.818 (1.0x)    |
| v0.3.6 vs 1.0.0 | 10    | 1         | 0.1       | 3.859   | 4.051   | 3.958   | 3.637 (1.1x)    | 3.694 (1.1x)    | 3.660 (1.1x)    |
| v0.3.6 vs 1.0.0 | 20    | 1         | 0.1       | 3.906   | 4.037   | 3.947   | 3.672 (1.1x)    | 3.798 (1.1x)    | 3.727 (1.1x)    |
| v0.3.6 vs 1.0.0 | 30    | 1         | 0.1       | 3.995   | 4.159   | 4.057   | 3.724 (1.1x)    | 3.877 (1.1x)    | 3.790 (1.1x)    |
| v0.3.6 vs 1.0.0 | 100   | 1         | 0.1       | 4.548   | 4.704   | 4.611   | 4.302 (1.1x)    | 4.469 (1.1x)    | 4.376 (1.1x)    |
| v0.3.6 vs 1.0.0 | 1000  | 1         | 0.1       | 6.776   | 6.921   | 6.851   | 6.434 (1.1x)    | 6.578 (1.1x)    | 6.503 (1.1x)    |
| v0.3.6 vs 1.0.0 | 10    | 2         | 0.1       | 1.999   | 2.063   | 2.030   | 1.861 (1.1x)    | 1.941 (1.1x)    | 1.890 (1.1x)    |
| v0.3.6 vs 1.0.0 | 20    | 2         | 0.1       | 2.039   | 2.109   | 2.055   | 1.924 (1.1x)    | 1.988 (1.1x)    | 1.939 (1.1x)    |
| v0.3.6 vs 1.0.0 | 30    | 2         | 0.1       | 2.066   | 2.138   | 2.086   | 1.954 (1.1x)    | 2.016 (1.1x)    | 1.973 (1.1x)    |
| v0.3.6 vs 1.0.0 | 100   | 2         | 0.1       | 2.888   | 3.010   | 2.950   | 2.258 (1.3x)    | 2.314 (1.3x)    | 2.277 (1.3x)    |
| v0.3.6 vs 1.0.0 | 1000  | 2         | 0.1       | 5.941   | 6.056   | 5.990   | 3.447 (1.7x)    | 3.526 (1.7x)    | 3.479 (1.7x)    |
| v0.3.6 vs 1.0.0 | 10    | 4         | 0.1       | 1.035   | 1.064   | 1.048   | 0.959 (1.1x)    | 0.997 (1.1x)    | 0.985 (1.1x)    |
| v0.3.6 vs 1.0.0 | 20    | 4         | 0.1       | 1.062   | 1.092   | 1.075   | 0.990 (1.1x)    | 1.017 (1.1x)    | 0.999 (1.1x)    |
| v0.3.6 vs 1.0.0 | 30    | 4         | 0.1       | 1.075   | 1.111   | 1.086   | 1.007 (1.1x)    | 1.044 (1.1x)    | 1.019 (1.1x)    |
| v0.3.6 vs 1.0.0 | 100   | 4         | 0.1       | 1.182   | 1.211   | 1.189   | 1.169 (1.0x)    | 1.195 (1.0x)    | 1.177 (1.0x)    |
| v0.3.6 vs 1.0.0 | 1000  | 4         | 0.1       | 1.602   | 1.633   | 1.611   | 1.821 (-1.1x)   | 1.859 (-1.1x)   | 1.831 (-1.1x)   |
| v0.3.6 vs 1.0.0 | 10    | 8         | 0.1       | 0.536   | 0.564   | 0.551   | 0.484 (1.1x)    | 0.504 (1.1x)    | 0.495 (1.1x)    |
| v0.3.6 vs 1.0.0 | 20    | 8         | 0.1       | 0.551   | 0.586   | 0.566   | 0.501 (1.1x)    | 0.513 (1.1x)    | 0.504 (1.1x)    |
| v0.3.6 vs 1.0.0 | 30    | 8         | 0.1       | 0.557   | 0.595   | 0.574   | 0.509 (1.1x)    | 0.527 (1.1x)    | 0.515 (1.1x)    |
| v0.3.6 vs 1.0.0 | 100   | 8         | 0.1       | 0.606   | 0.630   | 0.621   | 0.592 (1.0x)    | 0.613 (1.0x)    | 0.599 (1.0x)    |
| v0.3.6 vs 1.0.0 | 1000  | 8         | 0.1       | 0.838   | 0.867   | 0.851   | 0.968 (-1.2x)   | 0.993 (-1.1x)   | 0.981 (-1.2x)   |
| v0.3.6 vs 1.0.0 | 10    | 1         | 0.8       | 3.463   | 3.697   | 3.604   | 3.448 (1.0x)    | 3.581 (1.0x)    | 3.545 (1.0x)    |
| v0.3.6 vs 1.0.0 | 20    | 1         | 0.8       | 3.462   | 3.649   | 3.505   | 3.445 (1.0x)    | 3.614 (1.0x)    | 3.504 (1.0x)    |
| v0.3.6 vs 1.0.0 | 30    | 1         | 0.8       | 3.463   | 3.627   | 3.556   | 3.448 (1.0x)    | 3.586 (1.0x)    | 3.516 (1.0x)    |
| v0.3.6 vs 1.0.0 | 100   | 1         | 0.8       | 3.459   | 3.594   | 3.542   | 3.458 (1.0x)    | 3.623 (-1.0x)   | 3.539 (1.0x)    |
| v0.3.6 vs 1.0.0 | 1000  | 1         | 0.8       | 3.463   | 3.631   | 3.546   | 3.537 (-1.0x)   | 3.701 (-1.0x)   | 3.615 (-1.0x)   |
| v0.3.6 vs 1.0.0 | 10    | 2         | 0.8       | 1.808   | 1.895   | 1.844   | 1.806 (1.0x)    | 1.881 (1.0x)    | 1.852 (-1.0x)   |
| v0.3.6 vs 1.0.0 | 20    | 2         | 0.8       | 1.810   | 1.902   | 1.833   | 1.807 (1.0x)    | 1.887 (1.0x)    | 1.843 (-1.0x)   |
| v0.3.6 vs 1.0.0 | 30    | 2         | 0.8       | 1.817   | 1.892   | 1.874   | 1.814 (1.0x)    | 1.876 (1.0x)    | 1.856 (1.0x)    |
| v0.3.6 vs 1.0.0 | 100   | 2         | 0.8       | 2.894   | 3.017   | 2.981   | 1.821 (1.6x)    | 1.882 (1.6x)    | 1.860 (1.6x)    |
| v0.3.6 vs 1.0.0 | 1000  | 2         | 0.8       | 5.986   | 6.051   | 6.008   | 1.886 (3.2x)    | 1.944 (3.1x)    | 1.924 (3.1x)    |
| v0.3.6 vs 1.0.0 | 10    | 4         | 0.8       | 0.933   | 0.975   | 0.951   | 0.934 (-1.0x)   | 0.973 (1.0x)    | 0.958 (-1.0x)   |
| v0.3.6 vs 1.0.0 | 20    | 4         | 0.8       | 0.935   | 0.974   | 0.952   | 0.935 (-1.0x)   | 0.967 (1.0x)    | 0.947 (1.0x)    |
| v0.3.6 vs 1.0.0 | 30    | 4         | 0.8       | 0.954   | 0.980   | 0.969   | 0.941 (1.0x)    | 0.964 (1.0x)    | 0.957 (1.0x)    |
| v0.3.6 vs 1.0.0 | 100   | 4         | 0.8       | 0.939   | 0.975   | 0.964   | 0.946 (-1.0x)   | 0.969 (1.0x)    | 0.963 (1.0x)    |
| v0.3.6 vs 1.0.0 | 1000  | 4         | 0.8       | 0.951   | 0.991   | 0.970   | 0.975 (-1.0x)   | 1.001 (-1.0x)   | 0.996 (-1.0x)   |
| v0.3.6 vs 1.0.0 | 10    | 8         | 0.8       | 0.469   | 0.486   | 0.474   | 0.470 (-1.0x)   | 0.488 (-1.0x)   | 0.482 (-1.0x)   |
| v0.3.6 vs 1.0.0 | 20    | 8         | 0.8       | 0.472   | 0.494   | 0.475   | 0.469 (1.0x)    | 0.484 (1.0x)    | 0.477 (-1.0x)   |
| v0.3.6 vs 1.0.0 | 30    | 8         | 0.8       | 0.477   | 0.492   | 0.487   | 0.471 (1.0x)    | 0.483 (1.0x)    | 0.479 (1.0x)    |
| v0.3.6 vs 1.0.0 | 100   | 8         | 0.8       | 0.477   | 0.488   | 0.485   | 0.474 (1.0x)    | 0.488 (-1.0x)   | 0.483 (1.0x)    |
| v0.3.6 vs 1.0.0 | 1000  | 8         | 0.8       | 0.500   | 0.525   | 0.512   | 0.494 (1.0x)    | 0.509 (1.0x)    | 0.504 (1.0x)    |
